//Adrian Fiergolski EIKPE
//Przyklad uycia Multimapy - dokadny opis znajduje sie na stronie WIKI
//Proram wykorzysta multimape do operacji na miesicach roku

#include <string>
#include <map>	//multimap
#include <iostream>
#include <iterator> //ostream_iterator

using namespace std;

// Alias multimapy przechowujcej nazwy miesica wedug ich dugoci. Kluczem jest int - dugo miesica 
// w dniach; za przechowywan wartoci string - nazwa miesica. Trzeci parametr okrela obiekt funkcyjny
// porwnujcy dwa klucze w celu nadania porzdku w kontenerze. Domylnie jest to funktor less<T>, w przy-
// kadzie zostanie zastosowany greater<T> - miesice bd przechowywane od najduszego do najkrtszego.
// Multimapa bdzie korzysta z domylnego allocatora

typedef multimap<int, string, greater<int> > months_type;

// operatory << naley doczy do przestrzeni nazw std, w przeciwnym razie nie byyby wykorzystywane z powodu
// "Koenig Lookup" => www.gotw.ca/gotw/030.htm

namespace std
{
	// Szablon operatora << wypisujcy na standardowym wyjciu nazwe miesica i jego dugo, na podstawie pair<T,T>
	template <class First, class Second>   
		ostream& operator<<(ostream& out, const pair<First,Second>& p)
		{
			cout << p.second << " ma " << p.first << " dni";
			return out;
		}

	// wywietlenie zawartoci multimapy na standardowe wyjcie
	ostream& operator<<(ostream& out, months_type l)
	{
		// wykorzystanie copy z biblioteki stl,ktry kopiuje pewien zakres multimapy do ostram_iterator.
		// ostream_iterator jest iteratorem wyjcia (umoliwia tylko zapis wartoci do danego ska-
		// dnika  odczyt jest niemoliwy). Jego szablon wymaga zdefiniowania typu przechowywanych
		// elementw w strumieniu. Konstruktor ostream_iterator wywoywane jest z parametrami, ktre
		// definiuj strumie oraz char* dodawany na kocu kadego elementu. 
		copy(l.begin(),l.end(), ostream_iterator<months_type::value_type>(cout,"\n"));
		return out;
	}
}
int main(void)
{
    // tworzenie multimapy miesicy wedug ich dugci
    months_type months;

    //alias typu obiektu, pair<const key_type, data_type>, magazynowanego w multimapie
    typedef months_type::value_type value_type;

    // Dodawanie miesicy do multimapy: pierwszy w parze jest klucz, nastpnie sama warto elementu
    months.insert(value_type(31, string("Styczen")));
    months.insert(value_type(28, string("Luty")));
    months.insert(value_type(31, string("Marzec")));
    months.insert(value_type(30, string("Kwiecien")));
    months.insert(value_type(31, string("Maj")));
    months.insert(value_type(30, string("Czerwiec")));
    months.insert(value_type(31, string("Lipiec")));
    months.insert(value_type(31, string("Sierpien")));
    months.insert(value_type(30, string("Wrzesien")));
    months.insert(value_type(31, string("Pazdziernik")));
    months.insert(value_type(30, string("Listopad")));
    months.insert(value_type(31, string("Grudzien")));


    // Wywietlenie miesicy - naley zauwayc,e bd one posortowanie wedug zadanego wczeniej
    // funktora - od najduszego do najkrtszego
    cout << "Wszystkie miesiace roku ("<< months.size()<<"):" << endl << months << endl;

    // Wyszukanie miesicy z 30 dniami (wyszukiwanie elementw o zadanym kluczu i zwrcenie pary iteratorw
    // okrelajcych zakres wynikw wyszukiwania)
    pair<months_type::iterator,months_type::iterator> p = months.equal_range(30);

    // Wywietlenie znalezionych miesicy - wykorzystanie copy
    cout << endl << "Miesiace z 30 dniami" << endl;
    copy(p.first,p.second, ostream_iterator<months_type::value_type>(cout,"\n"));

    return 0;
}


